@@ -1,3 +1,5 @@
+from vorta.store.models import RepoModel
+
 from .borg_job import BorgJob
 
 
@@ -5,9 +7,17 @@ class BorgDeleteJob(BorgJob):
 
     def started_event(self):
         self.app.backup_started_event.emit()
-        self.app.backup_progress_event.emit(self.tr('Deleting archive...'))
+        self.app.backup_progress_event.emit(self.tr('Deleting archive…'))
 
     def finished_event(self, result):
+        # set repo stats to N/A
+        repo = RepoModel.get(id=result['params']['repo_id'])
+        repo.total_size = None
+        repo.unique_csize = None
+        repo.unique_size = None
+        repo.total_unique_chunks = None
+        repo.save()
+
         self.app.backup_finished_event.emit(result)
         self.result.emit(result)
         self.app.backup_progress_event.emit(self.tr('Archive deleted.'))
