@@ -1,48 +1,61 @@
+import logging
 import os.path
 import sys
 from datetime import timedelta
+from typing import Dict, Optional
 
 from PyQt5 import QtCore, uic
-from PyQt5.QtGui import QDesktopServices
-from PyQt5.QtWidgets import (QHeaderView, QMessageBox, QTableView,
-                             QTableWidgetItem, QInputDialog, QMenu,
-                             QToolButton)
+from PyQt5.QtCore import QItemSelectionModel, QMimeData, QPoint, Qt, pyqtSlot
+from PyQt5.QtGui import QDesktopServices, QKeySequence
+from PyQt5.QtWidgets import (QAction, QApplication, QHeaderView, QInputDialog,
+                             QLayout, QMenu, QMessageBox, QShortcut,
+                             QTableView, QTableWidgetItem, QWidget)
 
 from vorta.borg.check import BorgCheckJob
+from vorta.borg.compact import BorgCompactJob
 from vorta.borg.delete import BorgDeleteJob
 from vorta.borg.diff import BorgDiffJob
 from vorta.borg.extract import BorgExtractJob
+from vorta.borg.info_archive import BorgInfoArchiveJob
 from vorta.borg.list_archive import BorgListArchiveJob
 from vorta.borg.list_repo import BorgListRepoJob
-from vorta.borg.info_archive import BorgInfoArchiveJob
 from vorta.borg.mount import BorgMountJob
 from vorta.borg.prune import BorgPruneJob
-from vorta.borg.umount import BorgUmountJob
 from vorta.borg.rename import BorgRenameJob
+from vorta.borg.umount import BorgUmountJob
 from vorta.i18n import trans_late
 from vorta.store.models import ArchiveModel, BackupProfileMixin
 from vorta.utils import (choose_file_dialog, format_archive_name, get_asset,
                          get_mount_points, pretty_bytes)
-from vorta.views.source_tab import SizeItem
-from vorta.views.diff_dialog import DiffDialog
 from vorta.views.diff_result import DiffResult
 from vorta.views.extract_dialog import ExtractDialog
+from vorta.views.source_tab import SizeItem
 from vorta.views.utils import get_colored_icon
 
 uifile = get_asset('UI/archivetab.ui')
 ArchiveTabUI, ArchiveTabBase = uic.loadUiType(uifile)
 
+logger = logging.getLogger(__name__)
+
 
 class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
     prune_intervals = ['hour', 'day', 'week', 'month', 'year']
 
     def __init__(self, parent=None, app=None):
+        """Init."""
         super().__init__(parent)
         self.setupUi(parent)
-        self.mount_points = {}
+        self.mount_points = {}  # mount points of archives
+        self.repo_mount_point: Optional[
+            str] = None  # mount point of whole repo
         self.menu = None
         self.app = app
         self.toolBox.setCurrentIndex(0)
+        self.repoactions_enabled = True
+
+        #: Tooltip dict to save the tooltips set in the designer
+        self.tooltip_dict: Dict[QWidget, str] = {}
+        self.tooltip_dict[self.bDiff] = self.bDiff.toolTip()
 
         header = self.archiveTable.horizontalHeader()
         header.setVisible(True)
@@ -57,28 +70,49 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
             self._set_status('')  # Set platform-specific hints.
 
         self.archiveTable.setSelectionBehavior(QTableView.SelectRows)
-        self.archiveTable.setSelectionMode(QTableView.SingleSelection)
         self.archiveTable.setEditTriggers(QTableView.NoEditTriggers)
         self.archiveTable.setWordWrap(False)
         self.archiveTable.setTextElideMode(QtCore.Qt.ElideLeft)
         self.archiveTable.setAlternatingRowColors(True)
         self.archiveTable.cellDoubleClicked.connect(self.cell_double_clicked)
         self.archiveTable.setSortingEnabled(True)
+        self.archiveTable.setContextMenuPolicy(
+            Qt.ContextMenuPolicy.CustomContextMenu)
+        self.archiveTable.customContextMenuRequested.connect(
+            self.archiveitem_contextmenu)
 
-        self.listButton.clicked.connect(self.list_action)
-        self.pruneButton.clicked.connect(self.prune_action)
-        self.checkButton.clicked.connect(self.check_action)
-        self.diffButton.clicked.connect(self.diff_action)
+        # shortcuts
+        shortcut_copy = QShortcut(QKeySequence.StandardKey.Copy,
+                                  self.archiveTable)
+        shortcut_copy.activated.connect(self.archive_copy)
 
-        self.archiveActionMenu = QMenu(parent=self)
-        self.archiveActionMenu.aboutToShow.connect(self.showArchiveActionMenu)
-        self.archiveActionButton.setMenu(self.archiveActionMenu)
-        self.archiveActionButton.setPopupMode(QToolButton.InstantPopup)
+        # single and double selection feature
+        self.archiveTable.setSelectionMode(
+            QTableView.SelectionMode.ExtendedSelection)
+        self.archiveTable.selectionModel().selectionChanged.connect(
+            self.on_selection_change)
+
+        # connect archive actions
+        self.bMountArchive.clicked.connect(self.bmountarchive_clicked)
+        self.bRefreshArchive.clicked.connect(self.refresh_archive_info)
+        self.bRename.clicked.connect(self.rename_action)
+        self.bDelete.clicked.connect(self.delete_action)
+        self.bExtract.clicked.connect(self.extract_action)
+        self.compactButton.clicked.connect(self.compact_action)
+
+        # other signals
+        self.bList.clicked.connect(self.refresh_archive_list)
+        self.bPrune.clicked.connect(self.prune_action)
+        self.bCheck.clicked.connect(self.check_action)
+        self.bDiff.clicked.connect(self.diff_action)
+        self.bMountRepo.clicked.connect(self.bmountrepo_clicked)
 
         self.archiveNameTemplate.textChanged.connect(
-            lambda tpl, key='new_archive_name': self.save_archive_template(tpl, key))
+            lambda tpl, key='new_archive_name': self.save_archive_template(
+                tpl, key))
         self.prunePrefixTemplate.textChanged.connect(
-            lambda tpl, key='prune_prefix': self.save_archive_template(tpl, key))
+            lambda tpl, key='prune_prefix': self.save_archive_template(
+                tpl, key))
 
         self.populate_from_profile()
         self.selected_archives = None
@@ -86,13 +120,74 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
 
     def set_icons(self):
         "Used when changing between light- and dark mode"
-        self.checkButton.setIcon(get_colored_icon('check-circle'))
-        self.diffButton.setIcon(get_colored_icon('stream-solid'))
-        self.pruneButton.setIcon(get_colored_icon('cut'))
-        self.listButton.setIcon(get_colored_icon('refresh'))
+        self.bCheck.setIcon(get_colored_icon('check-circle'))
+        self.bDiff.setIcon(get_colored_icon('stream-solid'))
+        self.bPrune.setIcon(get_colored_icon('cut'))
+        self.bList.setIcon(get_colored_icon('refresh'))
+        self.compactButton.setIcon(get_colored_icon('broom-solid'))
         self.toolBox.setItemIcon(0, get_colored_icon('tasks'))
         self.toolBox.setItemIcon(1, get_colored_icon('cut'))
-        self.archiveActionButton.setIcon(get_colored_icon('ellipsis-v'))
+        self.bRefreshArchive.setIcon(get_colored_icon('refresh'))
+        self.bRename.setIcon(get_colored_icon('edit'))
+        self.bDelete.setIcon(get_colored_icon('trash'))
+        self.bExtract.setIcon(get_colored_icon('cloud-download'))
+
+        self.bmountarchive_refresh()
+        self.bmountrepo_refresh()
+
+    @pyqtSlot(QPoint)
+    def archiveitem_contextmenu(self, pos: QPoint):
+        # index under cursor
+        index = self.archiveTable.indexAt(pos)
+        if not index.isValid():
+            return  # popup only for items
+
+        selected_rows = self.archiveTable.selectionModel().selectedRows(
+            index.column())
+
+        if selected_rows and index not in selected_rows:
+            return  # popup only for selected items
+
+        menu = QMenu(self.archiveTable)
+        menu.addAction(get_colored_icon('copy'), self.tr("Copy"),
+                       lambda: self.archive_copy(index=index))
+        menu.addSeparator()
+
+        # archive actions
+        archive_actions = []
+        archive_actions.append(
+            menu.addAction(self.bRefreshArchive.icon(),
+                           self.bRefreshArchive.text(),
+                           self.refresh_archive_info))
+        archive_actions.append(
+            menu.addAction(self.bMount.icon(), self.bMount.text(),
+                           self.bmount_clicked))
+        archive_actions.append(
+            menu.addAction(self.bExtract.icon(), self.bExtract.text(),
+                           self.extract_action))
+        archive_actions.append(
+            menu.addAction(self.bRename.icon(), self.bRename.text(),
+                           self.rename_action))
+        archive_actions.append(
+            menu.addAction(self.bDelete.icon(), self.bDelete.text(),
+                           self.delete_action))
+
+        if not (self.repoactions_enabled and len(selected_rows) <= 1):
+            for action in archive_actions:
+                action.setEnabled(False)
+
+        # diff action
+        menu.addSeparator()
+        diff_action = QAction(self.bDiff.icon(), self.bDiff.text(), menu)
+        diff_action.triggered.connect(self.diff_action)
+        menu.addAction(diff_action)
+
+        selected_rows = self.archiveTable.selectionModel().selectedRows(
+            index.column())
+        diff_action.setEnabled(self.repoactions_enabled
+                               and len(selected_rows) > 1)
+
+        menu.popup(self.archiveTable.viewport().mapToGlobal(pos))
 
     def cancel_action(self):
         self._set_status(self.tr("Action cancelled."))
@@ -103,44 +198,33 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
         self.mountErrors.repaint()
 
     def _toggle_all_buttons(self, enabled=True):
-        for button in [self.checkButton, self.listButton, self.pruneButton,
-                       self.diffButton, self.archiveActionButton]:
+        """
+        Set all the buttons in the archive panel to the given state.
+
+        Parameters
+        ----------
+        enabled : bool, optional
+            The enabled state, by default True
+        """
+        self.repoactions_enabled = enabled
+
+        for button in [self.bCheck, self.bList, self.bPrune,
+                       self.bDiff, self.fArchiveActions, self.bMountRepo]:
             button.setEnabled(enabled)
             button.repaint()
 
-    def showArchiveActionMenu(self):
-        archive_name = self.selected_archive_name()
-        menu = self.archiveActionMenu
-        menu.clear()
-
-        if not archive_name:
-            action = menu.addAction(self.tr("Select an archive first."))
-            action.setEnabled(False)
-            return menu
-
-        if archive_name in self.mount_points:
-            unmountAction = menu.addAction("Unmount", self.umount_action)
-            unmountAction.setIcon(get_colored_icon('eject'))
-        else:
-            mountAction = menu.addAction("Mount", self.mount_action)
-            mountAction.setIcon(get_colored_icon('folder-open'))
-
-        extractAction = menu.addAction("Extract", self.list_archive_action)
-        refreshAction = menu.addAction("Refresh", self.refresh_archive_action)
-        renameAction = menu.addAction("Rename", self.rename_action)
-        deleteAction = menu.addAction("Delete", self.delete_action)
-
-        extractAction.setIcon(get_colored_icon('cloud-download'))
-        refreshAction.setIcon(get_colored_icon('refresh'))
-        renameAction.setIcon(get_colored_icon('edit'))
-        deleteAction.setIcon(get_colored_icon('trash'))
-        return menu
+        # Restore states
+        self.on_selection_change()
 
     def populate_from_profile(self):
         """Populate archive list and prune settings from profile."""
         profile = self.profile()
         if profile.repo is not None:
-            self.mount_points = get_mount_points(profile.repo.url)
+            # get mount points
+            self.mount_points, repo_mount_points = get_mount_points(profile.repo.url)
+            if repo_mount_points:
+                self.repo_mount_point = repo_mount_points[0]
+
             self.toolBox.setItemText(0, self.tr('Archives for %s') % profile.repo.url)
             archives = [s for s in profile.repo.archives.select().order_by(ArchiveModel.time.desc())]
 
@@ -170,6 +254,8 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
             self.archiveTable.setSortingEnabled(sorting)
             item = self.archiveTable.item(0, 0)
             self.archiveTable.scrollToItem(item)
+
+            self.archiveTable.selectionModel().clearSelection()
             self._toggle_all_buttons(enabled=True)
         else:
             self.mount_points = {}
@@ -188,6 +274,91 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
         self.prune_keep_within.setText(profile.prune_keep_within)
         self.prune_keep_within.editingFinished.connect(self.save_prune_setting)
 
+    def on_selection_change(self, selected=None, deselected=None):
+        """
+        React to a change of the selection of the archiveTableView.
+
+        Enables or disables archive actions and the diff button.
+        Makes sure at maximum 2 rows are selected.
+
+        Parameters
+        ----------
+        selected : QItemSelection, optional
+            The new selection.
+        deselected : QItemSelection, optional
+            The previous selection.
+        """
+        # handle selection of more than 2 rows
+        selectionModel: QItemSelectionModel = self.archiveTable.selectionModel(
+        )
+        indexes = selectionModel.selectedRows()
+
+        # Toggle archive actions frame
+        layout: QLayout = self.fArchiveActions.layout()
+
+        # Make sure at maximum 2 rows are selected.
+        if len(indexes) > 2:
+            selectionModel.select(
+                indexes[0], QItemSelectionModel.SelectionFlag.Deselect
+                | QItemSelectionModel.SelectionFlag.Rows)
+            indexes = selectionModel.selectedRows()
+
+        # Toggle diff button
+        if len(indexes) >= 2:
+            # Enable diff button
+            self.bDiff.setEnabled(True)
+            self.bDiff.setToolTip(self.tooltip_dict.get(self.bDiff, ""))
+        else:
+            # disable diff button
+            self.bDiff.setEnabled(False)
+
+            tooltip = self.tooltip_dict[self.bDiff]
+            self.bDiff.setToolTip(tooltip + " " +
+                                  self.tr("(Select two archives)"))
+
+        if len(indexes) == 1:
+            # Enable archive actions
+            self.fArchiveActions.setEnabled(True)
+
+            for index in range(layout.count()):
+                widget = layout.itemAt(index).widget()
+                widget.setToolTip(self.tooltip_dict.get(widget, ""))
+
+            # refresh bMount for the selected archive
+            self.bmountarchive_refresh()
+        else:
+            # too few or too many selected.
+            self.fArchiveActions.setEnabled(False)
+
+            for index in range(layout.count()):
+                widget = layout.itemAt(index).widget()
+                tooltip = widget.toolTip()
+
+                tooltip = self.tooltip_dict.setdefault(widget, tooltip)
+                widget.setToolTip(tooltip + " " +
+                                  self.tr("(Select exactly one archive)"))
+
+    def archive_copy(self, index=None):
+        """
+        Copy an archive name to the clipboard.
+
+        Copies the first selected archive if no index is specified.
+        """
+        if index is None:
+            indexes = self.archiveTable.selectionModel().selectedRows()
+
+            if not indexes:
+                return
+
+            index = indexes[0]
+
+        archive_name = self.archiveTable.item(index.row(), 4).text()
+
+        data = QMimeData()
+        data.setText(archive_name)
+
+        QApplication.clipboard().setMimeData(data)
+
     def save_archive_template(self, tpl, key):
         profile = self.profile()
         try:
@@ -226,6 +397,20 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
         if result['returncode'] == 0:
             self._toggle_all_buttons(True)
 
+    def compact_action(self):
+        params = BorgCompactJob.prepare(self.profile())
+        if params['ok']:
+            job = BorgCompactJob(params['cmd'], params, self.profile().repo.id)
+            job.updated.connect(self._set_status)
+            job.result.connect(self.compact_result)
+            self._toggle_all_buttons(False)
+            self.app.jobs_manager.add_job(job)
+        else:
+            self._set_status(params['message'])
+
+    def compact_result(self, result):
+        self._toggle_all_buttons(True)
+
     def prune_action(self):
         params = BorgPruneJob.prepare(self.profile())
         if params['ok']:
@@ -240,11 +425,11 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
     def prune_result(self, result):
         if result['returncode'] == 0:
             self._set_status(self.tr('Pruning finished.'))
-            self.list_action()
+            self.refresh_archive_list()
         else:
             self._toggle_all_buttons(True)
 
-    def list_action(self):
+    def refresh_archive_list(self):
         params = BorgListRepoJob.prepare(self.profile())
         if params['ok']:
             job = BorgListRepoJob(params['cmd'], params, self.profile().repo.id)
@@ -261,18 +446,18 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
             self._set_status(self.tr('Refreshed archives.'))
             self.populate_from_profile()
 
-    def refresh_archive_action(self):
+    def refresh_archive_info(self):
         archive_name = self.selected_archive_name()
         if archive_name is not None:
             params = BorgInfoArchiveJob.prepare(self.profile(), archive_name)
             if params['ok']:
                 job = BorgInfoArchiveJob(params['cmd'], params, self.profile().repo.id)
                 job.updated.connect(self._set_status)
-                job.result.connect(self.refresh_archive_result)
+                job.result.connect(self.info_result)
                 self._toggle_all_buttons(False)
                 self.app.jobs_manager.add_job(job)
 
-    def refresh_archive_result(self, result):
+    def info_result(self, result):
         self._toggle_all_buttons(True)
         if result['returncode'] == 0:
             self._set_status(self.tr('Refreshed archive.'))
@@ -286,25 +471,103 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                 return archive_cell.text()
         return None
 
-    def mount_action(self):
+    def bmountarchive_clicked(self):
+        """
+        Handle `bMountArchive` being clicked.
+
+        Mount or umount the current archive depending on its current state.
+        """
+        archive_name = self.selected_archive_name()
+
+        if not archive_name:
+            logger.warning("Archive name of selection is empty.")
+            return
+
+        if archive_name in self.mount_points:
+            self.unmount_action(archive_name=archive_name)
+        else:
+            self.mount_action(archive_name=archive_name)
+
+    def bmountrepo_clicked(self):
+        """
+        Handle `bMountRepo` being clicked.
+
+        Mount or umount the repository depending on its current state.
+        """
+        if self.repo_mount_point:
+            self.unmount_action()
+        else:
+            self.mount_action()
+
+    def bmountarchive_refresh(self):
+        """
+        Update label, tooltip and state of `bMount`.
+
+        The new state depends on the mount status of the current archive.
+        This also updates the icon of the button.
+        """
+        archive_name = self.selected_archive_name()
+
+        if archive_name in self.mount_points:
+            self.bMountArchive.setText(self.tr("Unmount"))
+            self.bMountArchive.setIcon(get_colored_icon('eject'))
+            self.bMountArchive.setToolTip(
+                self.tr('Unmount the selected archive from the file system.'))
+        else:
+            self.bMountArchive.setText(self.tr("Mount…"))
+            self.bMountArchive.setIcon(get_colored_icon('folder-open'))
+            self.bMountRepo.setToolTip(
+                self.tr("Mount the selected archive " +
+                        "as a folder in the file system."))
+
+    def bmountrepo_refresh(self):
+        """
+        Update label, tooltip and state of `bMount`.
+
+        The new state depends on the mount status of the current archive.
+        This also updates the icon of the button.
+        """
+        if self.repo_mount_point:
+            self.bMountRepo.setText(self.tr("Unmount"))
+            self.bMountRepo.setToolTip(
+                self.tr('Unmount the repository from the file system.'))
+            self.bMountRepo.setIcon(get_colored_icon('eject'))
+        else:
+            self.bMountRepo.setText(self.tr("Mount…"))
+            self.bMountRepo.setIcon(get_colored_icon('folder-open'))
+            self.bMountRepo.setToolTip(
+                self.tr("Mount the repository as a folder in the file system."))
+
+    def mount_action(self, archive_name=None):
+        """
+        Mount an archive or the whole repository.
+
+        Opens a file chooser to let the user choose a mount point and starts
+        the borg job for mounting afterwards.
+
+        Parameters
+        ----------
+        archive_name : str, optional
+            The archive to mount or None, by default None
+        """
         profile = self.profile()
         params = BorgMountJob.prepare(profile)
         if not params['ok']:
             self._set_status(params['message'])
             return
 
-        # Conditions are met (borg binary available, etc)
-        archive_name = self.selected_archive_name()
         if archive_name:
+            # mount archive
             params['cmd'][-1] += f'::{archive_name}'
             params['current_archive'] = archive_name
+        # else mount complete repo
 
         def receive():
             mount_point = dialog.selectedFiles()
             if mount_point:
                 params['cmd'].append(mount_point[0])
-                if params.get('current_archive', False):
-                    self.mount_points[params['current_archive']] = mount_point[0]
+                params['mount_point'] = mount_point[0]
+
                 if params['ok']:
                     self._toggle_all_buttons(False)
                     job = BorgMountJob(params['cmd'], params, self.profile().repo.id)
@@ -316,18 +579,48 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
         dialog.open(receive)
 
     def mount_result(self, result):
-        self._toggle_all_buttons(True)
         if result['returncode'] == 0:
             self._set_status(self.tr('Mounted successfully.'))
+
+            mount_point = result['params']['mount_point']
+
             if result['params'].get('current_archive'):
+                # archive was mounted
+                archive_name = result['params']['current_archive']
+                self.mount_points[archive_name] = mount_point
+
+                # update column in table
                 archive_name = result['params']['current_archive']
                 row = self.row_of_archive(archive_name)
                 item = QTableWidgetItem(result['cmd'][-1])
                 self.archiveTable.setItem(row, 3, item)
 
-    def umount_action(self):
-        archive_name = self.selected_archive_name()
-        mount_point = self.mount_points.get(archive_name)
+                # update button
+                self.bmountarchive_refresh()
+            else:
+                # whole repo was mounted
+                self.repo_mount_point = mount_point
+                self.bmountrepo_refresh()
+
+        self._toggle_all_buttons(True)
+
+    def unmount_action(self, archive_name=None):
+        """
+        Unmount a (mounted) repository or archive.
+
+        If the target isn't mounted nothing happens.
+
+        Parameters
+        ----------
+        archive_name : str, optional
+            The archive to unmount, by default None
+        """
+        if archive_name:
+            # unmount a single archive
+            mount_point = self.mount_points.get(archive_name)
+        else:
+            # unmount the whole repository
+            mount_point = self.repo_mount_point
 
         if mount_point is not None:
             profile = self.profile()
@@ -336,10 +629,13 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                 self._set_status(params['message'])
                 return
 
-            params['current_archive'] = archive_name
+            if archive_name:
+                params['current_archive'] = archive_name
+            params['mount_point'] = mount_point
 
             if os.path.normpath(mount_point) in params['active_mount_points']:
                 params['cmd'].append(mount_point)
+
                 job = BorgUmountJob(params['cmd'], params, self.profile().repo.id)
                 job.updated.connect(self.mountErrors.setText)
                 job.result.connect(self.umount_result)
@@ -350,16 +646,30 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
 
     def umount_result(self, result):
         self._toggle_all_buttons(True)
-        archive_name = result['params']['current_archive']
+        archive_name = result['params'].get('current_archive')
+        mount_point = result['params']['mount_point']
+
         if result['returncode'] == 0:
             self._set_status(self.tr('Un-mounted successfully.'))
-            del self.mount_points[archive_name]
-            row = self.row_of_archive(archive_name)
-            item = QTableWidgetItem('')
-            self.archiveTable.setItem(row, 3, item)
+
+            if archive_name:
+                # unmount single archive
+                del self.mount_points[archive_name]
+                row = self.row_of_archive(archive_name)
+                item = QTableWidgetItem('')
+                self.archiveTable.setItem(row, 3, item)
+
+                # update button
+                self.bmountarchive_refresh()
+            else:
+                # unmount repo
+                self.repo_mount_point = None
+
+                self.bmountrepo_refresh()
         else:
-            self._set_status(self.tr('Unmounting failed. Make sure no programs are using {}').format(
-                self.mount_points.get(archive_name)))
+            self._set_status(
+                self.tr('Unmounting failed. Make sure no programs are using {}')
+                .format(mount_point))
 
     def save_prune_setting(self, new_value=None):
         profile = self.profile()
@@ -368,7 +678,7 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
         profile.prune_keep_within = self.prune_keep_within.text()
         profile.save()
 
-    def list_archive_action(self):
+    def extract_action(self):
         profile = self.profile()
 
         row_selected = self.archiveTable.selectionModel().selectedRows()
@@ -386,13 +696,13 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
 
                 job = BorgListArchiveJob(params['cmd'], params, self.profile().repo.id)
                 job.updated.connect(self.mountErrors.setText)
-                job.result.connect(self.list_archive_result)
+                job.result.connect(self.extract_list_result)
                 self.app.jobs_manager.add_job(job)
                 return job
         else:
             self._set_status(self.tr('Select an archive to restore first.'))
 
-    def list_archive_result(self, result):
+    def extract_list_result(self, result):
         self._set_status('')
         if result['returncode'] == 0:
             def process_result():
@@ -484,48 +794,63 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
             self._toggle_all_buttons(True)
 
     def diff_action(self):
-        def process_result():
-            if window.selected_archives:
-                self.selected_archives = window.selected_archives
-            archive_cell_newer = self.archiveTable.item(self.selected_archives[0], 4)
-            archive_cell_older = self.archiveTable.item(self.selected_archives[1], 4)
-            if archive_cell_older and archive_cell_newer:
-                archive_name_newer = archive_cell_newer.text()
-                archive_name_older = archive_cell_older.text()
-
-                params = BorgDiffJob.prepare(profile, archive_name_older, archive_name_newer)
-
-                if params['ok']:
-                    self._toggle_all_buttons(False)
-                    job = BorgDiffJob(params['cmd'], params, self.profile().repo.id)
-                    job.updated.connect(self.mountErrors.setText)
-                    job.result.connect(self.list_diff_result)
-                    self.app.jobs_manager.add_job(job)
-                else:
-                    self._set_status(params['message'])
+        """
+        Handle the diff button being clicked.
 
+        Exactly two archives must be selected in `archiveTable`. This is
+        usually enforced by `on_selection_change`.
+        """
+        selected_archives = self.archiveTable.selectionModel().selectedRows()
         profile = self.profile()
 
-        window = DiffDialog(self.archiveTable)
-        self._toggle_all_buttons(True)
-        window.setParent(self, QtCore.Qt.Sheet)
-        self._window = window  # for testing
-        window.show()
-        window.accepted.connect(process_result)
+        name1 = self.archiveTable.item(selected_archives[0].row(), 4).text()
+        name2 = self.archiveTable.item(selected_archives[1].row(), 4).text()
+
+        archive1, archive2 = (profile.repo.archives.select().where(
+            (ArchiveModel.name == name1)
+            | (ArchiveModel.name == name2)).order_by(ArchiveModel.time.desc()))
+
+        archive_name_newer = archive1.name
+        archive_name_older = archive2.name
+
+        # Start diff job
+        params = BorgDiffJob.prepare(profile, archive_name_older,
+                                     archive_name_newer)
+
+        if params['ok']:
+            self._toggle_all_buttons(False)
+            job = BorgDiffJob(params['cmd'], params, self.profile().repo.id)
+            job.updated.connect(self.mountErrors.setText)
+            job.result.connect(self.list_diff_result)
+            self.app.jobs_manager.add_job(job)
+        else:
+            self._set_status(params['message'])
 
     def list_diff_result(self, result):
+        """
+        Process the result of the `BorgDiffJob`.
+
+        The `BorgDiffJob` was initiated by `diff_action`.
+
+        Parameters
+        ----------
+        result : dict
+            The BorgJob result.
+        """
         self._set_status('')
         if result['returncode'] == 0:
-            archive_newer = ArchiveModel.get(name=result['params']['archive_name_newer'])
-            archive_older = ArchiveModel.get(name=result['params']['archive_name_older'])
-            window = DiffResult(result['data'], archive_newer, archive_older, result['params']['json_lines'])
+            archive_newer = ArchiveModel.get(
+                name=result['params']['archive_name_newer'])
+            archive_older = ArchiveModel.get(
+                name=result['params']['archive_name_older'])
+            window = DiffResult(result['data'], archive_newer, archive_older,
+                                result['params']['json_lines'])
             self._toggle_all_buttons(True)
             window.setParent(self, QtCore.Qt.Sheet)
             self._resultwindow = window  # for testing
             window.show()
 
     def rename_action(self):
-
         profile = self.profile()
         params = BorgRenameJob.prepare(profile)
         if not params['ok']:
